#include <stdio.h>
#include <stdlib.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include <string.h>
#include <stdlib.h>

#define SCREEN_WIDTH    400
#define SCREEN_HEIGHT   400

double table[2][SCREEN_HEIGHT][SCREEN_WIDTH];
double nextTable[2][SCREEN_HEIGHT][SCREEN_WIDTH];

double DA = 1.0;
double DB = 0.5;
double f = 0.055;
double k = 0.062;

double Laplacian(int tab, int x, int y)
{
    int xp[3];
    int yp[3];

    for (int i = -1; i <= 1; i++)
    {
       xp[i + 1] = (x + i +  SCREEN_WIDTH) % SCREEN_WIDTH;
       yp[i + 1] = (y + i + SCREEN_HEIGHT) % SCREEN_HEIGHT;
    }

    double sum1 = table[tab][yp[0]][xp[0]] +
                  table[tab][yp[0]][xp[2]] +
                  table[tab][yp[2]][xp[0]] +
                  table[tab][yp[2]][xp[2]];

    double sum2 = table[tab][yp[0]][xp[1]] +
                  table[tab][yp[1]][xp[0]] +
                  table[tab][yp[1]][xp[2]] +
                  table[tab][yp[2]][xp[1]];

    return 0.05 * sum1 + 0.2 * sum2 - table[tab][yp[1]][xp[1]];
}

float tim = 0;
int cnt = 0;
void update()
{
    sys.StartPerfCounter();
    for (int y = 0; y < SCREEN_HEIGHT; ++y)
        for (int x = 0; x < SCREEN_WIDTH; ++x)
        {
            double A = table[0][y][x];
            double B = table[1][y][x];

            nextTable[0][y][x] = A + DA * Laplacian(0, x, y) - A * B * B + f * (1.0 - A);
            nextTable[1][y][x] = B + DB * Laplacian(1, x, y) + A * B * B - (k + f) * B;
        }

    memcpy(table, nextTable, SCREEN_WIDTH * SCREEN_HEIGHT * 2 * sizeof(double));

    cnt++;
    if (cnt < 2000)
        tim += sys.StopPerfCounter();
    else if (cnt == 2000)
        printf("%f\n", tim);
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Reaction/Diffusion 3", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    // initialize the table
    for (int y = 0; y < SCREEN_HEIGHT; ++y)
        for (int x = 0; x < SCREEN_WIDTH; ++x)
        {
            table[0][y][x] = 1.0;
            table[1][y][x] = 0.0;
        }

    for (int y = 0; y < 10; ++y)
        for (int x = 0; x < 10; ++x)
        {
            int posX = SCREEN_WIDTH  / 2 + x - 5;
            int posY = SCREEN_HEIGHT / 2 + y - 5;
            table[1][posY][posX] = 1.0;
        }

    while (sys.isQuitRequested() == false)
    {
        // draw the current table
        for (int y = 0; y < SCREEN_HEIGHT; ++y)
            for (int x = 0; x < SCREEN_WIDTH; ++x)
            {
                Color col(table[0][y][x] * 255,
                          0,
                          table[1][y][x] * 255);
                gfx.setPixel(x, y, col);
            }

        gfx.render();
        sys.processEvents();

        // update the table
        update();
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
